/* GNIO - GLib Network Layer of GIO
 *
 * Copyright (C) 2008 Christian Kellner, Samuel Cormier-Iijima
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Christian Kellner <gicmo@gnome.org>
 *          Samuel Cormier-Iijima <sciyoshi@gmail.com>
 */

#ifndef G_TCP_CLIENT_H
#define G_TCP_CLIENT_H

#include <glib-object.h>
#include <gio/gio.h>

#include <gnio/ginetsocketaddress.h>
#include <gnio/gnetworkinputstream.h>
#include <gnio/gnetworkoutputstream.h>

G_BEGIN_DECLS

#define G_TYPE_TCP_CLIENT         (g_tcp_client_get_type ())
#define G_TCP_CLIENT(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), G_TYPE_TCP_CLIENT, GTcpClient))
#define G_TCP_CLIENT_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), G_TYPE_TCP_CLIENT, GTcpClientClass))
#define G_IS_TCP_CLIENT(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), G_TYPE_TCP_CLIENT))
#define G_IS_TCP_CLIENT_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), G_TYPE_TCP_CLIENT))
#define G_TCP_CLIENT_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), G_TYPE_TCP_CLIENT, GTcpClient))

typedef struct _GTcpClient          GTcpClient;
typedef struct _GTcpClientClass     GTcpClientClass;
typedef struct _GTcpClientPrivate   GTcpClientPrivate;

struct _GTcpClient
{
  GObject parent;

  GTcpClientPrivate *priv;
};

struct _GTcpClientClass
{
  GObjectClass parent_class;
};

GType                  g_tcp_client_get_type          (void) G_GNUC_CONST;

GTcpClient *           g_tcp_client_new               (const gchar *hostname,
                                                       gushort      port);

GTcpClient *           g_tcp_client_new_from_address  (GInetSocketAddress *address);

GNetworkInputStream *  g_tcp_client_get_input_stream  (GTcpClient *client);

GNetworkOutputStream * g_tcp_client_get_output_stream (GTcpClient *client);

gboolean               g_tcp_client_connect           (GTcpClient    *client,
                                                       GCancellable  *cancellable,
                                                       GError       **error);

void                   g_tcp_client_connect_async     (GTcpClient          *client,
                                                       GCancellable        *cancellable,
                                                       GAsyncReadyCallback  callback,
                                                       gpointer             user_data);

gboolean               g_tcp_client_connect_finish    (GTcpClient    *client,
                                                       GAsyncResult  *result,
                                                       GError       **error);

void                   g_tcp_client_close             (GTcpClient    *client);

G_END_DECLS

#endif /* G_TCP_CLIENT_H */
